@@ -29,11 +29,11 @@ BDPluginSpec* bd_plugin_spec_copy (BDPluginSpec *spec) {
  * Frees @spec.
  */
 void bd_plugin_spec_free (BDPluginSpec *spec) {
-    g_free (spec->so_name);
+    g_free ((gchar *) spec->so_name);
     g_free (spec);
 }
 
-GType bd_plugin_spec_get_type () {
+GType bd_plugin_spec_get_type (void) {
     static GType type = 0;
 
     if (G_UNLIKELY(type == 0)) {
