@@ -1,18 +1,18 @@
 /*
  * Copyright (C) 2014  Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  *
  * Author: Vratislav Podzimek <vpodzime@redhat.com>
  */
@@ -44,6 +44,9 @@ GQuark bd_btrfs_error_quark (void)
 }
 
 BDBtrfsDeviceInfo* bd_btrfs_device_info_copy (BDBtrfsDeviceInfo *info) {
+    if (info == NULL)
+        return NULL;
+
     BDBtrfsDeviceInfo *new_info = g_new0 (BDBtrfsDeviceInfo, 1);
 
     new_info->id = info->id;
@@ -55,11 +58,17 @@ BDBtrfsDeviceInfo* bd_btrfs_device_info_copy (BDBtrfsDeviceInfo *info) {
 }
 
 void bd_btrfs_device_info_free (BDBtrfsDeviceInfo *info) {
+    if (info == NULL)
+        return;
+
     g_free (info->path);
     g_free (info);
 }
 
 BDBtrfsSubvolumeInfo* bd_btrfs_subvolume_info_copy (BDBtrfsSubvolumeInfo *info) {
+    if  (info == NULL)
+        return NULL;
+
     BDBtrfsSubvolumeInfo *new_info = g_new0 (BDBtrfsSubvolumeInfo, 1);
 
     new_info->id = info->id;
@@ -70,11 +79,17 @@ BDBtrfsSubvolumeInfo* bd_btrfs_subvolume_info_copy (BDBtrfsSubvolumeInfo *info)
 }
 
 void bd_btrfs_subvolume_info_free (BDBtrfsSubvolumeInfo *info) {
+    if (info == NULL)
+        return;
+
     g_free (info->path);
     g_free (info);
 }
 
 BDBtrfsFilesystemInfo* bd_btrfs_filesystem_info_copy (BDBtrfsFilesystemInfo *info) {
+    if (info == NULL)
+        return NULL;
+
     BDBtrfsFilesystemInfo *new_info = g_new0 (BDBtrfsFilesystemInfo, 1);
 
     new_info->label = g_strdup (info->label);
@@ -86,6 +101,9 @@ BDBtrfsFilesystemInfo* bd_btrfs_filesystem_info_copy (BDBtrfsFilesystemInfo *inf
 }
 
 void bd_btrfs_filesystem_info_free (BDBtrfsFilesystemInfo *info) {
+    if (info == NULL)
+        return;
+
     g_free (info->label);
     g_free (info->uuid);
     g_free (info);
@@ -99,7 +117,7 @@ static GMutex deps_check_lock;
 #define DEPS_BTRFS_MASK (1 << DEPS_BTRFS)
 #define DEPS_LAST 1
 
-static UtilDep deps[DEPS_LAST] = {
+static const UtilDep deps[DEPS_LAST] = {
     {"btrfs", BTRFS_MIN_VERSION, NULL, "[Bb]trfs.* v([\\d\\.]+)"},
 };
 
@@ -107,7 +125,7 @@ static UtilDep deps[DEPS_LAST] = {
 #define MODULE_DEPS_BTRFS_MASK (1 << MODULE_DEPS_BTRFS)
 #define MODULE_DEPS_LAST 1
 
-static gchar* module_deps[MODULE_DEPS_LAST] = { "btrfs" };
+static const gchar*const module_deps[MODULE_DEPS_LAST] = { "btrfs" };
 
 
 /**
@@ -118,7 +136,7 @@ static gchar* module_deps[MODULE_DEPS_LAST] = { "btrfs" };
  * Function checking plugin's runtime dependencies.
  *
  */
-gboolean bd_btrfs_check_deps () {
+gboolean bd_btrfs_check_deps (void) {
     GError *error = NULL;
     guint i = 0;
     gboolean status = FALSE;
@@ -157,7 +175,7 @@ gboolean bd_btrfs_check_deps () {
  * library's initialization functions.**
  *
  */
-gboolean bd_btrfs_init () {
+gboolean bd_btrfs_init (void) {
     /* nothing to do here */
     return TRUE;
 };
@@ -169,7 +187,7 @@ gboolean bd_btrfs_init () {
  * library's functions that unload it.**
  *
  */
-void bd_btrfs_close () {
+void bd_btrfs_close (void) {
     /* nothing to do here */
 }
 
@@ -601,9 +619,7 @@ BDBtrfsDeviceInfo** bd_btrfs_list_devices (const gchar *device, GError **error)
                                   "path[ \\t]+(?P<path>\\S+)\n";
     GRegex *regex = NULL;
     GMatchInfo *match_info = NULL;
-    guint8 i = 0;
-    GPtrArray *dev_infos = g_ptr_array_new ();
-    BDBtrfsDeviceInfo** ret = NULL;
+    GPtrArray *dev_infos;
 
     if (!check_deps (&avail_deps, DEPS_BTRFS_MASK, deps, DEPS_LAST, &deps_check_lock, error) ||
         !check_module_deps (&avail_module_deps, MODULE_DEPS_BTRFS_MASK, module_deps, MODULE_DEPS_LAST, &deps_check_lock, error))
@@ -617,13 +633,16 @@ BDBtrfsDeviceInfo** bd_btrfs_list_devices (const gchar *device, GError **error)
     }
 
     success = bd_utils_exec_and_capture_output (argv, NULL, &output, error);
-    if (!success)
+    if (!success) {
+        g_regex_unref (regex);
         /* error is already populated from the previous call */
         return NULL;
+    }
 
     lines = g_strsplit (output, "\n", 0);
     g_free (output);
 
+    dev_infos = g_ptr_array_new ();
     for (line_p = lines; *line_p; line_p++) {
         success = g_regex_match (regex, *line_p, 0, &match_info);
         if (!success) {
@@ -636,21 +655,16 @@ BDBtrfsDeviceInfo** bd_btrfs_list_devices (const gchar *device, GError **error)
     }
 
     g_strfreev (lines);
+    g_regex_unref (regex);
 
     if (dev_infos->len == 0) {
         g_set_error (error, BD_BTRFS_ERROR, BD_BTRFS_ERROR_PARSE, "Failed to parse information about devices");
+        g_ptr_array_free (dev_infos, TRUE);
         return NULL;
     }
 
-    /* now create the return value -- NULL-terminated array of BDBtrfsDeviceInfo */
-    ret = g_new0 (BDBtrfsDeviceInfo*, dev_infos->len + 1);
-    for (i=0; i < dev_infos->len; i++)
-        ret[i] = (BDBtrfsDeviceInfo*) g_ptr_array_index (dev_infos, i);
-    ret[i] = NULL;
-
-    g_ptr_array_free (dev_infos, FALSE);
-
-    return ret;
+    g_ptr_array_add (dev_infos, NULL);
+    return (BDBtrfsDeviceInfo **) g_ptr_array_free (dev_infos, FALSE);
 }
 
 /**
@@ -682,7 +696,7 @@ BDBtrfsSubvolumeInfo** bd_btrfs_list_subvolumes (const gchar *mountpoint, gboole
     guint64 i = 0;
     guint64 y = 0;
     guint64 next_sorted_idx = 0;
-    GPtrArray *subvol_infos = g_ptr_array_new ();
+    GPtrArray *subvol_infos;
     BDBtrfsSubvolumeInfo* item = NULL;
     BDBtrfsSubvolumeInfo* swap_item = NULL;
     BDBtrfsSubvolumeInfo** ret = NULL;
@@ -706,11 +720,11 @@ BDBtrfsSubvolumeInfo** bd_btrfs_list_subvolumes (const gchar *mountpoint, gboole
 
     success = bd_utils_exec_and_capture_output (argv, NULL, &output, error);
     if (!success) {
+        g_regex_unref (regex);
         if (g_error_matches (*error,  BD_UTILS_EXEC_ERROR, BD_UTILS_EXEC_ERROR_NOOUT)) {
             /* no output -> no subvolumes */
-            ret = g_new0 (BDBtrfsSubvolumeInfo*, 1);
             g_clear_error (error);
-            return ret;
+            return g_new0 (BDBtrfsSubvolumeInfo*, 1);
         } else {
             /* error is already populated from the call above or simply no output*/
             return NULL;
@@ -720,6 +734,7 @@ BDBtrfsSubvolumeInfo** bd_btrfs_list_subvolumes (const gchar *mountpoint, gboole
     lines = g_strsplit (output, "\n", 0);
     g_free (output);
 
+    subvol_infos = g_ptr_array_new ();
     for (line_p = lines; *line_p; line_p++) {
         success = g_regex_match (regex, *line_p, 0, &match_info);
         if (!success) {
@@ -732,9 +747,11 @@ BDBtrfsSubvolumeInfo** bd_btrfs_list_subvolumes (const gchar *mountpoint, gboole
     }
 
     g_strfreev (lines);
+    g_regex_unref (regex);
 
     if (subvol_infos->len == 0) {
         g_set_error (error, BD_BTRFS_ERROR, BD_BTRFS_ERROR_PARSE, "Failed to parse information about subvolumes");
+        g_ptr_array_free (subvol_infos, TRUE);
         return NULL;
     }
 
@@ -810,21 +827,26 @@ BDBtrfsFilesystemInfo* bd_btrfs_filesystem_info (const gchar *device, GError **e
     }
 
     success = bd_utils_exec_and_capture_output (argv, NULL, &output, error);
-    if (!success)
+    if (!success) {
         /* error is already populated from the call above or just empty
            output */
+        g_regex_unref (regex);
         return NULL;
+    }
 
     success = g_regex_match (regex, output, 0, &match_info);
     if (!success) {
         g_regex_unref (regex);
         g_match_info_free (match_info);
+        g_free (output);
         return NULL;
     }
 
-    g_regex_unref (regex);
     ret = get_filesystem_info_from_match (match_info);
     g_match_info_free (match_info);
+    g_regex_unref (regex);
+
+    g_free (output);
 
     return ret;
 }
