@@ -122,7 +122,7 @@ static gboolean get_unmount_error_new (struct libmnt_context *cxt, int rc, const
         else {
             if (*buf == '\0')
                 g_set_error (error, BD_FS_ERROR, BD_FS_ERROR_FAIL,
-                             "Unknow error when unmounting %s", spec);
+                             "Unknown error when unmounting %s", spec);
             else
                 g_set_error (error, BD_FS_ERROR, BD_FS_ERROR_FAIL,
                              "%s", buf);
@@ -346,7 +346,7 @@ static gboolean get_mount_error_new (struct libmnt_context *cxt, int rc, MountAr
         else {
             if (*buf == '\0')
                 g_set_error (error, BD_FS_ERROR, BD_FS_ERROR_FAIL,
-                             "Unknow error when mounting %s", args->device ? args->device : args->mountpoint);
+                             "Unknown error when mounting %s", args->device ? args->device : args->mountpoint);
             else
                 g_set_error (error, BD_FS_ERROR, BD_FS_ERROR_FAIL,
                              "%s", buf);
@@ -531,10 +531,14 @@ static gboolean run_as_user (MountFunc func, MountArgs *args, uid_t run_as_uid,
 
                   channel = g_io_channel_unix_new (pipefd[0]);
                   if (g_io_channel_read_to_end (channel, &error_msg, &msglen, &local_error) != G_IO_STATUS_NORMAL) {
-                      g_set_error (error, BD_FS_ERROR, BD_FS_ERROR_FAIL,
-                                   "Error while reading error: %s (%d)",
-                                   local_error->message, local_error->code);
-                      g_clear_error (&local_error);
+                      if (local_error) {
+                          g_set_error (error, BD_FS_ERROR, BD_FS_ERROR_FAIL,
+                                       "Error while reading error: %s (%d)",
+                                       local_error->message, local_error->code);
+                          g_clear_error (&local_error);
+                      } else
+                          g_set_error (error, BD_FS_ERROR, BD_FS_ERROR_FAIL,
+                                       "Unknoen error while reading error.");
                       g_io_channel_unref (channel);
                       close (pipefd[0]);
                       return FALSE;
@@ -547,6 +551,7 @@ static gboolean run_as_user (MountFunc func, MountArgs *args, uid_t run_as_uid,
                       g_clear_error (&local_error);
                       g_io_channel_unref (channel);
                       close (pipefd[0]);
+                      g_free (error_msg);
                       return FALSE;
                   }
 
@@ -559,6 +564,7 @@ static gboolean run_as_user (MountFunc func, MountArgs *args, uid_t run_as_uid,
 
                   g_io_channel_unref (channel);
                   close (pipefd[0]);
+                  g_free (error_msg);
                   return FALSE;
               } else {
                   close (pipefd[0]);
@@ -600,6 +606,7 @@ gboolean bd_fs_unmount (const gchar *spec, gboolean lazy, gboolean force, const
     uid_t current_uid = -1;
     gid_t current_gid = -1;
     const BDExtraArg **extra_p = NULL;
+    gchar *endptr = NULL;
     MountArgs args;
 
     args.spec = spec;
@@ -615,19 +622,19 @@ gboolean bd_fs_unmount (const gchar *spec, gboolean lazy, gboolean force, const
     if (extra) {
         for (extra_p=extra; *extra_p; extra_p++) {
             if ((*extra_p)->opt && (g_strcmp0 ((*extra_p)->opt, "run_as_uid") == 0)) {
-                run_as_uid = g_ascii_strtoull ((*extra_p)->val, NULL, 0);
+                run_as_uid = g_ascii_strtoull ((*extra_p)->val, &endptr, 0);
 
                 /* g_ascii_strtoull returns 0 in case of error */
-                if (run_as_uid == 0 && (g_strcmp0 ((*extra_p)->opt, "0") != 0)) {
+                if (run_as_uid == 0 && endptr == (*extra_p)->val) {
                     g_set_error (error, BD_FS_ERROR, BD_FS_ERROR_FAIL,
                                  "Invalid specification of UID: '%s'", (*extra_p)->val);
                     return FALSE;
                 }
             } else if ((*extra_p)->opt && (g_strcmp0 ((*extra_p)->opt, "run_as_gid") == 0)) {
-                run_as_gid = g_ascii_strtoull ((*extra_p)->val, NULL, 0);
+                run_as_gid = g_ascii_strtoull ((*extra_p)->val, &endptr, 0);
 
                 /* g_ascii_strtoull returns 0 in case of error */
-                if (run_as_gid == 0 && (g_strcmp0 ((*extra_p)->opt, "0") != 0)) {
+                if (run_as_gid == 0 && endptr == (*extra_p)->val) {
                     g_set_error (error, BD_FS_ERROR, BD_FS_ERROR_FAIL,
                                  "Invalid specification of GID: '%s'", (*extra_p)->val);
                     return FALSE;
@@ -673,6 +680,7 @@ gboolean bd_fs_mount (const gchar *device, const gchar *mountpoint, const gchar
     uid_t current_uid = -1;
     gid_t current_gid = -1;
     const BDExtraArg **extra_p = NULL;
+    gchar *endptr = NULL;
     MountArgs args;
 
     args.device = device;
@@ -689,19 +697,19 @@ gboolean bd_fs_mount (const gchar *device, const gchar *mountpoint, const gchar
     if (extra) {
         for (extra_p=extra; *extra_p; extra_p++) {
             if ((*extra_p)->opt && (g_strcmp0 ((*extra_p)->opt, "run_as_uid") == 0)) {
-                run_as_uid = g_ascii_strtoull ((*extra_p)->val, NULL, 0);
+                run_as_uid = g_ascii_strtoull ((*extra_p)->val, &endptr, 0);
 
                 /* g_ascii_strtoull returns 0 in case of error */
-                if (run_as_uid == 0 && (g_strcmp0 ((*extra_p)->opt, "0") != 0)) {
+                if (run_as_uid == 0 && endptr == (*extra_p)->val) {
                     g_set_error (error, BD_FS_ERROR, BD_FS_ERROR_FAIL,
                                  "Invalid specification of UID: '%s'", (*extra_p)->val);
                     return FALSE;
                 }
             } else if ((*extra_p)->opt && (g_strcmp0 ((*extra_p)->opt, "run_as_gid") == 0)) {
-                run_as_gid = g_ascii_strtoull ((*extra_p)->val, NULL, 0);
+                run_as_gid = g_ascii_strtoull ((*extra_p)->val, &endptr, 0);
 
                 /* g_ascii_strtoull returns 0 in case of error */
-                if (run_as_gid == 0 && (g_strcmp0 ((*extra_p)->opt, "0") != 0)) {
+                if (run_as_gid == 0 && endptr == (*extra_p)->val) {
                     g_set_error (error, BD_FS_ERROR, BD_FS_ERROR_FAIL,
                                  "Invalid specification of GID: '%s'", (*extra_p)->val);
                     return FALSE;
