@@ -1,18 +1,18 @@
 /*
  * Copyright (C) 2018  Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  *
  * Author: Vojtech Trefny <vtrefny@redhat.com>
  */
@@ -43,6 +43,9 @@ GQuark bd_nvdimm_error_quark (void) {
 }
 
 void bd_nvdimm_namespace_info_free (BDNVDIMMNamespaceInfo *info) {
+    if (info == NULL)
+        return;
+
     g_free (info->dev);
     g_free (info->uuid);
     g_free (info->blockdev);
@@ -50,6 +53,9 @@ void bd_nvdimm_namespace_info_free (BDNVDIMMNamespaceInfo *info) {
 }
 
 BDNVDIMMNamespaceInfo* bd_nvdimm_namespace_info_copy (BDNVDIMMNamespaceInfo *info) {
+    if (info == NULL)
+        return NULL;
+
     BDNVDIMMNamespaceInfo *new_info = g_new0 (BDNVDIMMNamespaceInfo, 1);
 
     new_info->dev = g_strdup (info->dev);
@@ -64,7 +70,7 @@ BDNVDIMMNamespaceInfo* bd_nvdimm_namespace_info_copy (BDNVDIMMNamespaceInfo *inf
 }
 
 
-static const gchar * const mode_str[BD_NVDIMM_NAMESPACE_MODE_UNKNOWN+1] = {"raw", "sector", "memory", "dax", "unknown"};
+static const gchar * const mode_str[BD_NVDIMM_NAMESPACE_MODE_UNKNOWN+1] = {"raw", "sector", "memory", "dax", "fsdax", "devdax", "unknown"};
 
 static volatile guint avail_deps = 0;
 static GMutex deps_check_lock;
@@ -73,7 +79,7 @@ static GMutex deps_check_lock;
 #define DEPS_NDCTL_MASK (1 << DEPS_NDCTL)
 #define DEPS_LAST 1
 
-static UtilDep deps[DEPS_LAST] = {
+static const UtilDep deps[DEPS_LAST] = {
     {"ndctl", NULL, NULL, NULL},
 };
 
@@ -85,7 +91,7 @@ static UtilDep deps[DEPS_LAST] = {
  * Function checking plugin's runtime dependencies.
  *
  */
-gboolean bd_nvdimm_check_deps () {
+gboolean bd_nvdimm_check_deps (void) {
     GError *error = NULL;
     guint i = 0;
     gboolean status = FALSE;
@@ -115,7 +121,7 @@ gboolean bd_nvdimm_check_deps () {
  * library's initialization functions.**
  *
  */
-gboolean bd_nvdimm_init () {
+gboolean bd_nvdimm_init (void) {
     /* nothing to do here */
     return TRUE;
 }
@@ -127,7 +133,7 @@ gboolean bd_nvdimm_init () {
  * library's functions that unload it.**
  *
  */
-void bd_nvdimm_close () {
+void bd_nvdimm_close (void) {
     /* nothing to do here */
     return;
 }
@@ -178,6 +184,10 @@ BDNVDIMMNamespaceMode bd_nvdimm_namespace_get_mode_from_str (const gchar *mode_s
         return BD_NVDIMM_NAMESPACE_MODE_MEMORY;
     else if (g_strcmp0 (mode_str, "dax") == 0)
         return BD_NVDIMM_NAMESPACE_MODE_DAX;
+    else if (g_strcmp0 (mode_str, "fsdax") == 0)
+        return BD_NVDIMM_NAMESPACE_MODE_FSDAX;
+    else if (g_strcmp0 (mode_str, "devdax") == 0)
+        return BD_NVDIMM_NAMESPACE_MODE_DEVDAX;
     else {
         g_set_error (error, BD_NVDIMM_ERROR, BD_NVDIMM_ERROR_NAMESPACE_MODE_INVAL,
                      "Invalid mode given: '%s'", mode_str);
@@ -389,7 +399,11 @@ static BDNVDIMMNamespaceInfo* get_nvdimm_namespace_info (struct ndctl_namespace
               info->size = ndctl_pfn_get_size (pfn);
             else
               info->size = ndctl_namespace_get_size (ndns);
+#ifndef LIBNDCTL_NEW_MODES
           info->mode = BD_NVDIMM_NAMESPACE_MODE_MEMORY;
+#else
+          info->mode = BD_NVDIMM_NAMESPACE_MODE_FSDAX;
+#endif
           break;
         case NDCTL_NS_MODE_DAX:
             if (!dax) {
@@ -400,7 +414,11 @@ static BDNVDIMMNamespaceInfo* get_nvdimm_namespace_info (struct ndctl_namespace
                 return NULL;
             }
             info->size = ndctl_dax_get_size (dax);
+#ifndef LIBNDCTL_NEW_MODES
             info->mode = BD_NVDIMM_NAMESPACE_MODE_DAX;
+#else
+            info->mode = BD_NVDIMM_NAMESPACE_MODE_DEVDAX;
+#endif
             break;
         case NDCTL_NS_MODE_SAFE:
             if (!btt) {
@@ -618,3 +636,38 @@ gboolean bd_nvdimm_namespace_reconfigure (const gchar* namespace, BDNVDIMMNamesp
     g_free ((gchar *) args[5]);
     return ret;
 }
+
+
+static guint64 blk_sector_sizes[] = { 512, 520, 528, 4096, 4104, 4160, 4224, 0 };
+static guint64 pmem_sector_sizes[] = { 512, 4096, 0 };
+static guint64 io_sector_sizes[] = { 0 };
+
+/**
+ * bd_nvdimm_namepace_get_supported_sector_sizes:
+ * @mode: namespace mode
+ * @error: (out): place to store error if any
+ *
+ * Returns: (transfer none) (array zero-terminated=1): list of supported sector sizes for @mode
+ *
+ * Tech category: %BD_NVDIMM_TECH_NAMESPACE-%BD_NVDIMM_TECH_MODE_QUERY
+ */
+const guint64 *bd_nvdimm_namepace_get_supported_sector_sizes (BDNVDIMMNamespaceMode mode, GError **error) {
+    switch (mode) {
+        case BD_NVDIMM_NAMESPACE_MODE_RAW:
+        case BD_NVDIMM_NAMESPACE_MODE_MEMORY:
+        case BD_NVDIMM_NAMESPACE_MODE_FSDAX:
+            return pmem_sector_sizes;
+
+        case BD_NVDIMM_NAMESPACE_MODE_DAX:
+        case BD_NVDIMM_NAMESPACE_MODE_DEVDAX:
+            return io_sector_sizes;
+
+        case BD_NVDIMM_NAMESPACE_MODE_SECTOR:
+            return blk_sector_sizes;
+
+        default:
+            g_set_error (error, BD_NVDIMM_ERROR, BD_NVDIMM_ERROR_NAMESPACE_MODE_INVAL,
+                         "Invalid/unknown mode specified.");
+            return NULL;
+    }
+}
