@@ -22,9 +22,12 @@
 #include "extra_arg.h"
 #include <syslog.h>
 #include <stdlib.h>
+#include <poll.h>
+#include <fcntl.h>
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <unistd.h>
 
 #ifdef __clang__
 #define ZERO_INIT {}
@@ -139,11 +142,6 @@ static void log_done (guint64 task_id, gint exit_code) {
     return;
 }
 
-static void set_c_locale(gpointer user_data __attribute__((unused))) {
-    if (setenv ("LC_ALL", "C", 1) != 0)
-        g_warning ("Failed to set LC_ALL=C for a child process!");
-}
-
 /**
  * bd_utils_exec_and_report_error:
  * @argv: (array zero-terminated=1): the argv array for the call
@@ -193,6 +191,8 @@ gboolean bd_utils_exec_and_report_status_error (const gchar **argv, const BDExtr
     const BDExtraArg **extra_p = NULL;
     gint exit_status = 0;
     guint i = 0;
+    gchar **old_env = NULL;
+    gchar **new_env = NULL;
 
     if (extra) {
         args_len = g_strv_length ((gchar **) argv);
@@ -218,16 +218,20 @@ gboolean bd_utils_exec_and_report_status_error (const gchar **argv, const BDExtr
         args[i] = NULL;
     }
 
+    old_env = g_get_environ ();
+    new_env = g_environ_setenv (old_env, "LC_ALL", "C", TRUE);
+
     task_id = log_running (args ? args : argv);
-    success = g_spawn_sync (NULL, args ? (gchar **) args : (gchar **) argv, NULL, G_SPAWN_SEARCH_PATH,
-                            (GSpawnChildSetupFunc) set_c_locale, NULL,
-                            &stdout_data, &stderr_data, &exit_status, error);
+    success = g_spawn_sync (NULL, args ? (gchar **) args : (gchar **) argv, new_env, G_SPAWN_SEARCH_PATH,
+                            NULL, NULL, &stdout_data, &stderr_data, &exit_status, error);
     if (!success) {
         /* error is already populated from the call */
+        g_strfreev (new_env);
         g_free (stdout_data);
         g_free (stderr_data);
         return FALSE;
     }
+    g_strfreev (new_env);
 
     /* g_spawn_sync set the status in the same way waitpid() does, we need
        to get the process exit code manually (this is similar to calling
@@ -250,13 +254,6 @@ gboolean bd_utils_exec_and_report_status_error (const gchar **argv, const BDExtr
 
     g_free (args);
 
-    if (!success) {
-        /* error is already populated from the call */
-        g_free (stdout_data);
-        g_free (stderr_data);
-        return FALSE;
-    }
-
     if (*status != 0) {
         if (stderr_data && (g_strcmp0 ("", stderr_data) != 0)) {
             g_set_error (error, BD_UTILS_EXEC_ERROR, BD_UTILS_EXEC_ERROR_FAILED,
@@ -276,115 +273,88 @@ gboolean bd_utils_exec_and_report_status_error (const gchar **argv, const BDExtr
     return TRUE;
 }
 
-/**
- * bd_utils_exec_and_capture_output:
- * @argv: (array zero-terminated=1): the argv array for the call
- * @extra: (allow-none) (array zero-terminated=1): extra arguments
- * @output: (out): variable to store output to
- * @error: (out): place to store error (if any)
- *
- * Returns: whether the @argv was successfully executed capturing the output or not
- */
-gboolean bd_utils_exec_and_capture_output (const gchar **argv, const BDExtraArg **extra, gchar **output, GError **error) {
-    gchar *stdout_data = NULL;
-    gchar *stderr_data = NULL;
-    gint status = 0;
-    gint exit_status = 0;
-    gboolean success = FALSE;
-    guint64 task_id = 0;
-    const gchar **args = NULL;
-    guint args_len = 0;
-    const gchar **arg_p = NULL;
-    const BDExtraArg **extra_p = NULL;
-    guint i = 0;
+/* buffer size in bytes used to read from stdout and stderr */
+#define _EXEC_BUF_SIZE 64*1024
 
-    if (extra) {
-        args_len = g_strv_length ((gchar **) argv);
-        for (extra_p=extra; *extra_p; extra_p++) {
-            if ((*extra_p)->opt && (g_strcmp0 ((*extra_p)->opt, "") != 0))
-                args_len++;
-            if ((*extra_p)->val && (g_strcmp0 ((*extra_p)->val, "") != 0))
-                args_len++;
-        }
-        args = g_new0 (const gchar*, args_len + 1);
-        for (arg_p=argv; *arg_p; arg_p++, i++)
-            args[i] = *arg_p;
-        for (extra_p=extra; *extra_p; extra_p++) {
-            if ((*extra_p)->opt && (g_strcmp0 ((*extra_p)->opt, "") != 0)) {
-                args[i] = (*extra_p)->opt;
-                i++;
-            }
-            if ((*extra_p)->val && (g_strcmp0 ((*extra_p)->val, "") != 0)) {
-                args[i] = (*extra_p)->val;
-                i++;
+/* similar to g_strstr_len() yet treats 'null' byte as @needle. */
+static gchar *bd_strchr_len_null (const gchar *haystack, gssize haystack_len, const gchar needle) {
+    gchar *ret;
+    gchar *ret_null;
+
+    ret = memchr (haystack, needle, haystack_len);
+    ret_null = memchr (haystack, 0, haystack_len);
+    if (ret && ret_null)
+        return MIN (ret, ret_null);
+    else
+        return MAX (ret, ret_null);
+}
+
+static gboolean
+_process_fd_event (gint fd, struct pollfd *poll_fd, GString *read_buffer, GString *filtered_buffer, gsize *read_buffer_pos, gboolean *done,
+                   guint64 progress_id, guint8 *progress, BDUtilsProgExtract prog_extract, GError **error) {
+    gchar buf[_EXEC_BUF_SIZE] = { 0 };
+    ssize_t num_read;
+    gchar *line;
+    gchar *newline_pos;
+    int errno_saved;
+    gboolean eof = FALSE;
+
+    if (! *done && (poll_fd->revents & POLLIN)) {
+        /* read until we get EOF (0) or error (-1), expecting EAGAIN */
+        while ((num_read = read (fd, buf, _EXEC_BUF_SIZE)) > 0)
+            g_string_append_len (read_buffer, buf, num_read);
+        errno_saved = errno;
+
+        /* process the fresh data by lines */
+        if (read_buffer->len > *read_buffer_pos) {
+            gchar *buf_ptr;
+            gsize buf_len;
+
+            while ((buf_ptr = read_buffer->str + *read_buffer_pos,
+                    buf_len = read_buffer->len - *read_buffer_pos,
+                    newline_pos = bd_strchr_len_null (buf_ptr, buf_len, '\n'))) {
+                line = g_strndup (buf_ptr, newline_pos - buf_ptr + 1);
+                if (prog_extract && prog_extract (line, progress))
+                    bd_utils_report_progress (progress_id, *progress, NULL);
+                else
+                    g_string_append (filtered_buffer, line);
+                g_free (line);
+                *read_buffer_pos = newline_pos - read_buffer->str + 1;
             }
         }
-        args[i] = NULL;
-    }
 
-    task_id = log_running (argv);
-    success = g_spawn_sync (NULL, (gchar **) argv, NULL, G_SPAWN_SEARCH_PATH,
-                            (GSpawnChildSetupFunc) set_c_locale, NULL,
-                            &stdout_data, &stderr_data, &exit_status, error);
-
-    if (!success) {
-        /* error is already populated */
-        g_free (stdout_data);
-        g_free (stderr_data);
-        return FALSE;
-    }
-
-    /* g_spawn_sync set the status in the same way waitpid() does, we need
-       to get the process exit code manually (this is similar to calling
-       WEXITSTATUS but also sets the error for terminated processes */
-    if (!g_spawn_check_exit_status (exit_status, error)) {
-        if (g_error_matches (*error, G_SPAWN_ERROR, G_SPAWN_ERROR_FAILED)) {
-            /* process was terminated abnormally (e.g. using a signal) */
-            g_free (stdout_data);
-            g_free (stderr_data);
+        /* read error */
+        if (num_read < 0 && errno_saved != EAGAIN && errno_saved != EINTR) {
+            g_set_error (error, BD_UTILS_EXEC_ERROR, BD_UTILS_EXEC_ERROR_FAILED,
+                         "Error reading from pipe: %s", g_strerror (errno_saved));
             return FALSE;
         }
 
-        status = (*error)->code;
-        g_clear_error (error);
-    } else
-        status = 0;
-
-
-    log_out (task_id, stdout_data, stderr_data);
-    log_done (task_id, status);
-
-    if ((status != 0) || (g_strcmp0 ("", stdout_data) == 0)) {
-        if (status != 0)
-            g_set_error (error, BD_UTILS_EXEC_ERROR, BD_UTILS_EXEC_ERROR_FAILED,
-                         "Process reported exit code %d: %s%s", status,
-                         stdout_data ? stdout_data : "",
-                         stderr_data ? stderr_data : "");
-        else
-            g_set_error (error, BD_UTILS_EXEC_ERROR, BD_UTILS_EXEC_ERROR_NOOUT,
-                         "Process didn't provide any data on standard output. "
-                         "Error output: %s", stderr_data ? stderr_data : "");
-        g_free (stderr_data);
-        g_free (stdout_data);
-        return FALSE;
-    } else {
-        *output = stdout_data;
-        g_free (stderr_data);
-        return TRUE;
+        /* EOF */
+        if (num_read == 0)
+            eof = TRUE;
     }
+
+    if (poll_fd->revents & POLLHUP || poll_fd->revents & POLLERR || poll_fd->revents & POLLNVAL)
+        eof = TRUE;
+
+    if (eof) {
+        *done = TRUE;
+        /* process the remaining buffer */
+        line = read_buffer->str + *read_buffer_pos;
+        /* GString guarantees the buffer is always NULL-terminated. */
+        if (strlen (line) > 0) {
+            if (prog_extract && prog_extract (line, progress))
+                bd_utils_report_progress (progress_id, *progress, NULL);
+            else
+                g_string_append (filtered_buffer, line);
+        }
+    }
+
+    return TRUE;
 }
 
-/**
- * bd_utils_exec_and_report_progress:
- * @argv: (array zero-terminated=1): the argv array for the call
- * @extra: (allow-none) (array zero-terminated=1): extra arguments
- * @prog_extract: (scope notified): function for extracting progress information
- * @proc_status: (out): place to store the process exit status
- * @error: (out): place to store error (if any)
- *
- * Returns: whether the @argv was successfully executed (no error and exit code 0) or not
- */
-gboolean bd_utils_exec_and_report_progress (const gchar **argv, const BDExtraArg **extra, BDUtilsProgExtract prog_extract, gint *proc_status, GError **error) {
+static gboolean _utils_exec_and_report_progress (const gchar **argv, const BDExtraArg **extra, BDUtilsProgExtract prog_extract, gint *proc_status, gchar **stdout, gchar **stderr, GError **error) {
     const gchar **args = NULL;
     guint args_len = 0;
     const gchar **arg_p = NULL;
@@ -395,21 +365,26 @@ gboolean bd_utils_exec_and_report_progress (const gchar **argv, const BDExtraArg
     gchar *msg = NULL;
     GPid pid = 0;
     gint out_fd = 0;
-    GIOChannel *out_pipe = NULL;
     gint err_fd = 0;
-    GIOChannel *err_pipe = NULL;
-    gchar *line = NULL;
     gint child_ret = -1;
     gint status = 0;
     gboolean ret = FALSE;
-    GIOStatus io_status = G_IO_STATUS_NORMAL;
+    gint poll_status = 0;
     guint i = 0;
     guint8 completion = 0;
-    GPollFD fds[2] = {ZERO_INIT, ZERO_INIT};
+    struct pollfd fds[2] = { ZERO_INIT, ZERO_INIT };
+    int flags;
     gboolean out_done = FALSE;
     gboolean err_done = FALSE;
-    GString *stdout_data = g_string_new (NULL);
-    GString *stderr_data = g_string_new (NULL);
+    GString *stdout_data;
+    GString *stdout_buffer;
+    GString *stderr_data;
+    GString *stderr_buffer;
+    gsize stdout_buffer_pos = 0;
+    gsize stderr_buffer_pos = 0;
+    gchar **old_env = NULL;
+    gchar **new_env = NULL;
+    gboolean success = TRUE;
 
     /* TODO: share this code between functions */
     if (extra) {
@@ -438,10 +413,15 @@ gboolean bd_utils_exec_and_report_progress (const gchar **argv, const BDExtraArg
 
     task_id = log_running (args ? args : argv);
 
-    ret = g_spawn_async_with_pipes (NULL, args ? (gchar**) args : (gchar**) argv, NULL,
+    old_env = g_get_environ ();
+    new_env = g_environ_setenv (old_env, "LC_ALL", "C", TRUE);
+
+    ret = g_spawn_async_with_pipes (NULL, args ? (gchar**) args : (gchar**) argv, new_env,
                                     G_SPAWN_DEFAULT|G_SPAWN_SEARCH_PATH|G_SPAWN_DO_NOT_REAP_CHILD,
                                     NULL, NULL, &pid, NULL, &out_fd, &err_fd, error);
 
+    g_strfreev (new_env);
+
     if (!ret) {
         /* error is already populated */
         g_free (args);
@@ -455,126 +435,162 @@ gboolean bd_utils_exec_and_report_progress (const gchar **argv, const BDExtraArg
     g_free (args);
     g_free (msg);
 
-    out_pipe = g_io_channel_unix_new (out_fd);
-    err_pipe = g_io_channel_unix_new (err_fd);
+    /* set both fds for non-blocking read */
+    flags = fcntl (out_fd, F_GETFL, 0);
+    if (fcntl (out_fd, F_SETFL, flags | O_NONBLOCK))
+        g_warning ("_utils_exec_and_report_progress: Failed to set out_fd non-blocking: %m");
+    flags = fcntl (err_fd, F_GETFL, 0);
+    if (fcntl (err_fd, F_SETFL, flags | O_NONBLOCK))
+        g_warning ("_utils_exec_and_report_progress: Failed to set err_fd non-blocking: %m");
+
+    stdout_data = g_string_new (NULL);
+    stdout_buffer = g_string_new (NULL);
+    stderr_data = g_string_new (NULL);
+    stderr_buffer = g_string_new (NULL);
 
     fds[0].fd = out_fd;
     fds[1].fd = err_fd;
-    fds[0].events = G_IO_IN | G_IO_HUP | G_IO_ERR;
-    fds[1].events = G_IO_IN | G_IO_HUP | G_IO_ERR;
-    while (!out_done || !err_done) {
-        g_poll (fds, 2, -1);
-        if (!out_done && (fds[0].revents & (G_IO_IN | G_IO_HUP | G_IO_ERR))) {
-            if (fds[0].revents & G_IO_IN) {
-                io_status = g_io_channel_read_line (out_pipe, &line, NULL, NULL, error);
-                if (io_status == G_IO_STATUS_NORMAL) {
-                    if (prog_extract && prog_extract (line, &completion))
-                        bd_utils_report_progress (progress_id, completion, NULL);
-                    else
-                        g_string_append (stdout_data, line);
-                    g_free (line);
-                } else if (io_status == G_IO_STATUS_EOF) {
-                    out_done = TRUE;
-                } else if (error && (*error)) {
-                    bd_utils_report_finished (progress_id, (*error)->message);
-                    g_io_channel_shutdown (out_pipe, FALSE, NULL);
-                    g_io_channel_unref (out_pipe);
-                    g_io_channel_shutdown (err_pipe, FALSE, NULL);
-                    g_io_channel_unref (err_pipe);
-                    g_string_free (stdout_data, TRUE);
-                    g_string_free (stderr_data, TRUE);
-                    return FALSE;
-                }
-            } else
-                /* ERR or HUP, nothing more to do here */
-                out_done = TRUE;
+    fds[0].events = POLLIN | POLLHUP | POLLERR;
+    fds[1].events = POLLIN | POLLHUP | POLLERR;
+    while (! (out_done && err_done)) {
+        poll_status = poll (fds, 2, -1 /* timeout */);
+        g_warn_if_fail (poll_status != 0);  /* no timeout specified, zero should never be returned */
+        if (poll_status < 0) {
+            if (errno == EAGAIN || errno == EINTR)
+                continue;
+            g_set_error (error, BD_UTILS_EXEC_ERROR, BD_UTILS_EXEC_ERROR_FAILED,
+                         "Failed to poll output FDs: %m");
+            bd_utils_report_finished (progress_id, (*error)->message);
+            success = FALSE;
+            break;
         }
-        if (!err_done && (fds[1].revents & (G_IO_IN | G_IO_HUP | G_IO_ERR))) {
-            if (fds[1].revents & G_IO_IN) {
-                io_status = g_io_channel_read_line (err_pipe, &line, NULL, NULL, error);
-                if (io_status == G_IO_STATUS_NORMAL) {
-                    g_string_append (stderr_data, line);
-                    g_free (line);
-                } else if (io_status == G_IO_STATUS_EOF) {
-                    err_done = TRUE;
-                } else if (error && (*error)) {
-                    bd_utils_report_finished (progress_id, (*error)->message);
-                    g_io_channel_shutdown (out_pipe, FALSE, NULL);
-                    g_io_channel_unref (out_pipe);
-                    g_io_channel_shutdown (err_pipe, FALSE, NULL);
-                    g_io_channel_unref (err_pipe);
-                    g_string_free (stdout_data, TRUE);
-                    g_string_free (stderr_data, TRUE);
-                    return FALSE;
-                }
-            } else
-                /* ERR or HUP, nothing more to do here */
-                err_done = TRUE;
+
+        if (!out_done) {
+            if (! _process_fd_event (out_fd, &fds[0], stdout_buffer, stdout_data, &stdout_buffer_pos, &out_done, progress_id, &completion, prog_extract, error)) {
+                bd_utils_report_finished (progress_id, (*error)->message);
+                success = FALSE;
+                break;
+            }
+        }
+
+        if (!err_done) {
+            if (! _process_fd_event (err_fd, &fds[1], stderr_buffer, stderr_data, &stderr_buffer_pos, &err_done, progress_id, &completion, prog_extract, error)) {
+                bd_utils_report_finished (progress_id, (*error)->message);
+                success = FALSE;
+                break;
+            }
         }
     }
 
+    g_string_free (stdout_buffer, TRUE);
+    g_string_free (stderr_buffer, TRUE);
+    close (out_fd);
+    close (err_fd);
+
     child_ret = waitpid (pid, &status, 0);
-    *proc_status = WEXITSTATUS(status);
-    if (child_ret > 0) {
-        if (*proc_status != 0) {
-            if (stderr_data->str && (g_strcmp0 ("", stderr_data->str) != 0))
-                msg = stderr_data->str;
-            else
-                msg = stdout_data->str;
-            g_set_error (error, BD_UTILS_EXEC_ERROR, BD_UTILS_EXEC_ERROR_FAILED,
-                         "Process reported exit code %d: %s", *proc_status, msg);
-            bd_utils_report_finished (progress_id, (*error)->message);
-            g_io_channel_shutdown (out_pipe, FALSE, NULL);
-            g_io_channel_unref (out_pipe);
-            g_io_channel_shutdown (err_pipe, FALSE, NULL);
-            g_io_channel_unref (err_pipe);
-            g_string_free (stdout_data, TRUE);
-            g_string_free (stderr_data, TRUE);
-            return FALSE;
+    *proc_status = WEXITSTATUS (status);
+    if (success) {
+        if (child_ret > 0) {
+            if (*proc_status != 0) {
+                msg = stderr_data->len > 0 ? stderr_data->str : stdout_data->str;
+                g_set_error (error, BD_UTILS_EXEC_ERROR, BD_UTILS_EXEC_ERROR_FAILED,
+                             "Process reported exit code %d: %s", *proc_status, msg);
+                bd_utils_report_finished (progress_id, (*error)->message);
+                success = FALSE;
+            } else if (WIFSIGNALED (status)) {
+                g_set_error (error, BD_UTILS_EXEC_ERROR, BD_UTILS_EXEC_ERROR_FAILED,
+                             "Process killed with a signal");
+                bd_utils_report_finished (progress_id, (*error)->message);
+                success = FALSE;
+            }
+        } else if (child_ret == -1) {
+            if (errno != ECHILD) {
+                errno = 0;
+                g_set_error (error, BD_UTILS_EXEC_ERROR, BD_UTILS_EXEC_ERROR_FAILED,
+                             "Failed to wait for the process");
+                bd_utils_report_finished (progress_id, (*error)->message);
+                success = FALSE;
+            } else {
+                /* no such process (the child exited before we tried to wait for it) */
+                errno = 0;
+            }
         }
-        if (WIFSIGNALED(status)) {
-            g_set_error (error, BD_UTILS_EXEC_ERROR, BD_UTILS_EXEC_ERROR_FAILED,
-                         "Process killed with a signal");
-            bd_utils_report_finished (progress_id, (*error)->message);
-            g_io_channel_shutdown (out_pipe, FALSE, NULL);
-            g_io_channel_unref (out_pipe);
-            g_io_channel_shutdown (err_pipe, FALSE, NULL);
-            g_io_channel_unref (err_pipe);
-            g_string_free (stdout_data, TRUE);
-            g_string_free (stderr_data, TRUE);
-            return FALSE;
-        }
-    } else if (child_ret == -1) {
-        if (errno != ECHILD) {
-            errno = 0;
-            g_set_error (error, BD_UTILS_EXEC_ERROR, BD_UTILS_EXEC_ERROR_FAILED,
-                         "Failed to wait for the process");
-            bd_utils_report_finished (progress_id, (*error)->message);
-            g_io_channel_shutdown (out_pipe, FALSE, NULL);
-            g_io_channel_unref (out_pipe);
-            g_io_channel_shutdown (err_pipe, FALSE, NULL);
-            g_io_channel_unref (err_pipe);
-            g_string_free (stdout_data, TRUE);
-            g_string_free (stderr_data, TRUE);
-            return FALSE;
-        }
-        /* no such process (the child exited before we tried to wait for it) */
-        errno = 0;
+        if (success)
+            bd_utils_report_finished (progress_id, "Completed");
     }
-
-    bd_utils_report_finished (progress_id, "Completed");
     log_out (task_id, stdout_data->str, stderr_data->str);
     log_done (task_id, *proc_status);
 
-    /* we don't care about the status here */
-    g_io_channel_shutdown (out_pipe, FALSE, error);
-    g_io_channel_unref (out_pipe);
-    g_io_channel_shutdown (err_pipe, FALSE, error);
-    g_io_channel_unref (err_pipe);
-    g_string_free (stdout_data, TRUE);
-    g_string_free (stderr_data, TRUE);
+    if (success && stdout)
+        *stdout = g_string_free (stdout_data, FALSE);
+    else
+        g_string_free (stdout_data, TRUE);
+    if (success && stderr)
+        *stderr = g_string_free (stderr_data, FALSE);
+    else
+        g_string_free (stderr_data, TRUE);
 
-    return TRUE;
+    return success;
+}
+
+/**
+ * bd_utils_exec_and_report_progress:
+ * @argv: (array zero-terminated=1): the argv array for the call
+ * @extra: (allow-none) (array zero-terminated=1): extra arguments
+ * @prog_extract: (scope notified) (nullable): function for extracting progress information
+ * @proc_status: (out): place to store the process exit status
+ * @error: (out): place to store error (if any)
+ *
+ * Note that any NULL bytes read from standard output and standard error
+ * output are treated as separators similar to newlines and @prog_extract
+ * will be called with the respective chunk.
+ *
+ * Returns: whether the @argv was successfully executed (no error and exit code 0) or not
+ */
+gboolean bd_utils_exec_and_report_progress (const gchar **argv, const BDExtraArg **extra, BDUtilsProgExtract prog_extract, gint *proc_status, GError **error) {
+    return _utils_exec_and_report_progress (argv, extra, prog_extract, proc_status, NULL, NULL, error);
+}
+
+/**
+ * bd_utils_exec_and_capture_output:
+ * @argv: (array zero-terminated=1): the argv array for the call
+ * @extra: (allow-none) (array zero-terminated=1): extra arguments
+ * @output: (out): variable to store output to
+ * @error: (out): place to store error (if any)
+ *
+ * Note that any NULL bytes read from standard output and standard error
+ * output will be discarded.
+ *
+ * Returns: whether the @argv was successfully executed capturing the output or not
+ */
+gboolean bd_utils_exec_and_capture_output (const gchar **argv, const BDExtraArg **extra, gchar **output, GError **error) {
+    gint status = 0;
+    gchar *stdout = NULL;
+    gchar *stderr = NULL;
+    gboolean ret = FALSE;
+
+    ret = _utils_exec_and_report_progress (argv, extra, NULL, &status, &stdout, &stderr, error);
+    if (!ret)
+        return ret;
+
+    if ((status != 0) || (g_strcmp0 ("", stdout) == 0)) {
+        if (status != 0)
+            g_set_error (error, BD_UTILS_EXEC_ERROR, BD_UTILS_EXEC_ERROR_FAILED,
+                         "Process reported exit code %d: %s%s", status,
+                         stdout ? stdout : "",
+                         stderr ? stderr : "");
+        else
+            g_set_error (error, BD_UTILS_EXEC_ERROR, BD_UTILS_EXEC_ERROR_NOOUT,
+                         "Process didn't provide any data on standard output. "
+                         "Error output: %s", stderr ? stderr : "");
+        g_free (stderr);
+        g_free (stdout);
+        return FALSE;
+    } else {
+        *output = stdout;
+        g_free (stderr);
+        return TRUE;
+    }
 }
 
 /**
